import click
import os
import json
from maxar_canvas_grid import Cell, covers


@click.group()
def cell():
    ''' Subcommand for ARD Grid cell tools '''

    pass


    
@click.command()
@click.argument('quadkey', required=True)
@click.argument('zone', required=False)
@click.option('--format', help='Output the cell in different formats: `geojson` for GeoJSON')
def describe(quadkey, zone, format):
    ''' Output a description of a cell given a QUADKEY and optionally a ZONE

        QUADKEY by itself must be a full Cell ID: z15-230232110
        or
        QUADKEY is just the quadkey: 230232110 plus a ZONE : 15 '''

    if zone is None:
        cell = Cell(quadkey)
    else:
        cell = Cell(quadkey, zone=zone)
    # error handling
    if format is None:
        print(f'Cell {cell.quadkey} in zone {cell.zone}')
        xmin, ymin, xmax, ymax = cell.bounds
        print ('UTM bounds: ')
        print(f' Lower left corner: {xmin}, {ymin}')
        print(f' Upper right corner: {xmax}, {ymax}')
        print ('Geographic bounds:')
        xmin, ymin, xmax, ymax = cell.geom_WGS84.bounds
        print(f' Lower left corner: {xmin}, {ymin}')
        print(f' Upper right corner: {xmax}, {ymax}')

        print(f'Cell size: {xmax - xmin}m')
    if format == 'geojson':
        print(cell.to_GeoJSON())


@click.command(name='covers')
@click.argument('geom', type=str)
@click.option('--zoom', type=int, help='Zoom of cells to calculate, default is 12')
@click.option('--format', help='Output the cellis in different formats: `geojson` for GeoJSON')
def _covers(geom, format, zoom):
    ''' Calculate the cell IDs that cover a given geometry GEOM. GEOM can be WKT or GeoJSON '''

    if zoom is None:
        cells = covers(geom)
    else:
        cells = covers(geom, zoom=zoom)
    if format is None:
        for cell in cells:
            print(cell.id)
    if format == 'geojson':
        fc = {
            "type": "FeatureCollection",
            "features": []
        }
        for cell in cells:
            fc['features'].append(cell.to_feature())
        print(json.dumps(fc))

cell.add_command(_covers)
cell.add_command(describe)